/* Control the LEDs on the WNR3500L Router
 * (C) David Schueler, 2010, david.schueler@wapkamera.de
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA02111-1307USA
 *
 * Feedback, Bugs...  blogic@openwrt.org
 *
 */

#include <stdio.h>
#include <stdlib.h>
#include <sys/types.h>
#include <sys/stat.h>
#include <fcntl.h>
#include <linux/ioctl.h>

/* linux stuff */
typedef u_int64_t uint64;
typedef u_int32_t uint32;
typedef u_int16_t uint16;
typedef u_int8_t uint8;

#include <wnr_linux_gpio.h>

#define WLAN	0	/* WLAN led */
#define WPS		1	/* WPS button led */
#define WAN		2	/* i led (green if on, orange if off) */
#define PWRGRN	3	/* Power led (green if on) */
#define PWRRED	7	/* Power led (orange if on) */

void
print_usage()
{
	printf("ledctl power_orange|power_green|wan_green|wlan|wps on|off|1|0\n");
	exit(0);
}

int
main(int argc, char **argv)
{
	int fd, i;
	uint16 pin, sta;
	struct gpio_ioctl gpioioc;

	if (argc < 3) {
		print_usage();
	}

	if ((fd = open("/dev/gpio", O_RDWR)) < 0) {
        	printf("Error while opening /dev/gpio\n");
	        return -1;
	}

	if (strcmp(argv[1],"power_orange") == 0) {
		gpioioc.mask = 1 << PWRRED;
		gpioioc.val  = 1 << PWRRED;
	} else if (strcmp(argv[1],"power_green") == 0) {
		gpioioc.mask = 1 << PWRGRN;
		gpioioc.val  = 1 << PWRGRN;
	} else if (strcmp(argv[1],"wan_green") == 0) {
		gpioioc.mask = 1 << WAN;
		gpioioc.val  = 1 << WAN;
	} else if (strcmp(argv[1],"wlan") == 0) {
		gpioioc.mask = 1 << WLAN;
		gpioioc.val  = 1 << WLAN;
	} else if (strcmp(argv[1],"wps") == 0) {
		gpioioc.mask = 1 << WPS;
		gpioioc.val  = 1 << WPS;
	} else if (strcmp(argv[1],"direct") == 0) {
			gpioioc.mask = 1 << atoi(argv[3]);
			gpioioc.val  = 1 << atoi(argv[3]);
	} else {
		print_usage();
	}

	if(ioctl(fd, GPIO_IOC_RESERVE, &gpioioc) < 0)
		printf("Invalid gpioreg %d\n", GPIO_IOC_RESERVE);

	if (strcmp(argv[2],"1") == 0 || strcmp(argv[2],"on") == 0) {
		gpioioc.val = gpioioc.mask;
	} else {
		gpioioc.val = 0;
	}
	if(ioctl(fd, GPIO_IOC_OUT, &gpioioc))
		printf("Invalid gpioreg %d\n", GPIO_IOC_OUT);

	return 0;
}
